/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000;

import static java.lang.System.out;
import java.awt.Color;
import java.io.File;
import java.util.*;
import it.andynaz.log.*;
import it.andynaz.utils.Utils;
import it.andynaz.irex.dr1000.document.*;

/**
 * Main class.
 * 
 * <p>The scribbles can be taken on a PDF document or with the device built-in
 * notepad. In case of PDF documents, this program creates a new PDF with all
 * its interactive elements, with the scribbles on it. The device must have the
 * version&nbsp;2.0 of the firmware.
 * 
 * <p>To use this program run in a terminal</p>
 * <blockquote><pre>
 * java -jar iRexDR1000Merger.jar <i>options</i>
 * </pre></blockquote>
 * where <i>options</i> are in the form "<code>-<b>x</b> param</code>".
 * <p>Parameters are:</p>
 * <dl>
 *   <dt><b>f</b></dt>
 *   <dd>target PDF or {@code .note} file</dd>
 *   <dt><b>o</b></dt>
 *   <dd>output file (optional; if not present the default one is taken)</dd>
 *   <dt><b>a</b></dt>
 *   <dd>action to be performed (optional; the default one is 'merge')</dd>
 *   <dt><b>c</b></dt>
 *   <dd>path to the a properties file with the configuration of the program
 *   (optional; the default one is a {@code config.properties} file in the
 *   current directory)</dd>
 *   <dt><b>C</b></dt>
 *   <dd>color of the strokes in the resulting PDF file; the color is specified
 *   as a hex value {@code RRGGBB} (default is the black color)</dd>
 * </dl>
 * 
 * <p><b>Note</b>: in case the input file is a PDF, it has to be placed in the
 * SD card used by the DR1000, i.e. in the same folder of the {@code
 * metedata.bd} file.</p>
 * 
 * <p>Possible actions are:</p>
 * <dl>
 *   <dt><b>merge</b></dt>
 *   <dd>default action: merge the scribbles with the PDF in a new PDF file or
 *   creates a new PDF file with the note (default output file is a PDF file in
 *   the current folder)</dd>
 *   <dt><b>format</b></dt>
 *   <dd>create a text file with the scribbles data formatted (default output
 *   file is a {@code .dat} file in the current folder)</dd>
 * </dl>
 * 
 * <p>Optionally, you can invoke the program as</p>
 * <blockquote><pre>
 * java -jar iRexDR1000Merger.jar <i>path_to_file</i>
 * </pre></blockquote>
 * <p>in which you specify only the input file (other option use their default
 * value).</p>
 *
 * @author andynaz
 * @version 2013-08-01
 */
public class Merger {

   /**
	 * Takes the arguments from the command line and start the program.
	 * 
	 * @param args the command line arguments
	 * @throws Exception in case of errors
    */
   public static void main(String[] args) throws Exception {
		printHeader();
		
		if (args.length==1 && "-h".equals(args[0])){
			printHelp();
			return;
		}
		
		// --- config.properties: takes the 'c' parameter or the file 'config.properties'
		String configPath = Utils.getInitParam("c", args);
		configPath = Utils.isEmpty(configPath) ? "config.properties" : configPath;
		if( new File(configPath).exists() )
			Config.init(configPath);
		
		Logger logger = LoggerMgr.getLogger("main");
		logger.log(Arrays.toString(args), Level.FINEST);
		
		// check arguments: at least one
      if (args.length==0){
			logger.log("error: no arguments", Level.SEVERE);
			return;
		}
		
		// check if the program has been invoked with the only-one-argument syntax
		boolean oneArgOnly = args.length==1;
		
		// --- input file
		String inputFilePath = null;
		if (oneArgOnly){
			inputFilePath = args[0];
		} else{
			inputFilePath = Utils.getInitParam("f", args);
			if( inputFilePath==null ){
				logger.log("error: no input file", Level.SEVERE);
				return;
			}
		}
		logger.log("input file: "+inputFilePath, Level.DEBUG);
		
		// --- action
		String action = oneArgOnly ? "merge" : Utils.getInitParam("a", args);
		if (action==null) action="merge";
		logger.log("action: "+action, Level.DEBUG);
		
		// --- output file
		String outputFilePath = Utils.getInitParam("o", args);
		logger.log("output file: "+outputFilePath, Level.DEBUG);
		
		// --- stroke color
		String colorCode = Utils.getInitParam("C", args);
		Color color = ColorMgr.defaultColor;
		if (!Utils.isEmpty(colorCode))
			color = ColorMgr.decode(colorCode);
		logger.log("stroke color: "+color, Level.DEBUG);
		
		File inputFile = new File(inputFilePath);
		
		Document doc;
		if (inputFilePath.toLowerCase().endsWith(".pdf")){
			doc = new PdfDocument(inputFile);
		} else if (inputFilePath.toLowerCase().endsWith(".note")){
			doc = new Note(inputFile);
		} else{
			logger.log("error: no PDF file, filePath="+inputFilePath, Level.SEVERE);
			return;
		}
		
		if ("merge".equals(action)){
			doc.merge(outputFilePath, color);
		} else if ("format".equals(action)){
			doc.format(outputFilePath);
		}
   }

	/**
	 * Print the program header.
	 * 
	 * This contains also a licence advice.
	 */
	private static void printHeader() {
		out.println("--------------------------------------------------------------------------------");
		out.println("   iRex DR1000 Merger           Copyright (c) 2013  Andrea \"andynaz\" Agnesse");
		out.println();
		out.println(" This program comes with ABSOLUTELY NO WARRANTY.");
		out.println(" This is free software, and you are welcome to redistribute it under certain");
		out.println(" conditions; read the file 'licence.txt' in the distributed binary for details.");
		out.println("--------------------------------------------------------------------------------");
		out.println();
	}

	/**
	 * Print the program help.
	 */
	private static void printHelp() {
		out.println();
		out.println("This program let you merge the scribbles taken with an iRex DR1000 device on a");
		out.println("PDF, or creates a PDF from the notes.");
		out.println("The device needs to have the version 2 of the firmware.");
		out.println();
		out.println("SYNOPSIS");
		out.println("  java -jar iRexDR1000Merger-[version].jar [input file]");
		out.println("  java -jar iRexDR1000Merger-[version].jar -f input_file [options]");
		out.println();
		out.println("DESCRIPTION");
		out.println("  [input file]");
		out.println("    a PDF file path on a SD card used by the iRex DR1000 (the program looks for");
		out.println("    the 'metadata.db' file in the same directory of the PDF file)");
		out.println("  [options]");
		out.println("    couples of [-X value] options:");
		out.println("    f  target PDF or note file");
		out.println("    o  output file (optional)");
		out.println("    a  action to be performed (optional, default is 'merge')");
		out.println("    c  path to the a properties file with the configuration of the program");
		out.println("       (optional, default is a 'config.properties' file in the current");
		out.println("       directory)");
		out.println("    C  color of the strokes in the resulting PDF file; the color is specified as");
		out.println("       an 'RRGGBB' value (default is black)");
		out.println();
	}
	
}
