/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.document;

import java.awt.Color;

/**
 * Manage the colors.
 * 
 * Contains a default color to be used in the strokes and a method to safely
 * decode a color.
 * 
 * <p>Note that the color codes for this class can be specified both as a string
 * <code>RRGGBB</code> or <code>#RRGGBB</code>.</p>
 * 
 *
 * @author andynaz
 * @version 2013-04-23
 */
public class ColorMgr {
	/**
	 * Default color to use in strokes.
	 * 
	 * Can be set using the global property <code>-Dandynaz.defaultColor=<b>RRGGBB</b></code>;
	 * if it is not set or the color can not be decoded, is the {@link
	 * Color#BLACK black} color.
	 * 
	 * @see Color#decode(java.lang.String)
	 */
	public static final Color defaultColor;
	
	static{
		Color c;
		try{
			c = Color.decode(normalizeColorCode(System.getProperty("andynaz.defaultColor")));
		} catch(Exception e){
			c = Color.BLACK;
		}
		defaultColor = c;
	}
	
	/**
	 * Safely decode a color.
	 * 
	 * @param colorCode numeric code of the color, as specified in che {@link
	 * Color} class
	 * @return the color corresponding to the {@code colorCode}, or the {@link
	 * #defaultColor default color} if the code can not be decoded
	 */
	public static Color decode(String colorCode){
		Color c;
		try{
			c = Color.decode(normalizeColorCode(colorCode));
		} catch(Exception e){
			System.out.println(e);
			c = defaultColor;
		}
		return c;
	}
	
	/**
	 * A '#' character is added at the beginning of the string {@code s} if it do
	 * not begin with '#'.
	 * 
	 * @param s a string color code
	 * @return the same string suitable to be used with the method {@link
	 * Color#decode(java.lang.String)}
	 */
	private static String normalizeColorCode(String s) {
		return s.startsWith("#") ? s : '#'+s;
	}
	
}
