/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.document;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collections;
import java.util.List;
import it.andynaz.log.*;
import it.andynaz.irex.dr1000.scribble.Util;
import it.andynaz.utils.Utils;
import java.awt.Color;
import java.io.ByteArrayInputStream;

/**
 * Document on a iRex DR1000.
 * 
 * This class ia a model for both a PDF document or a note: it has an associated
 * SQLite database file, in which the scribbles are stored.
 *
 * @author andynaz
 * @version 2013-08-01
 */
public abstract class Document {
	
	/** Database file. */
	protected File db;
	
	
	/**
	 * Creates a new Document with an associated database file.
	 * 
	 * @param db database file
	 * 
	 * @throws IllegalArgumentException il the {@code db} file does not exists or
	 * is not a file
	 */
	protected Document(File db) {
		if (!db.exists()) throw new IllegalArgumentException("db file '"+db+"' does not exist");
		if (!db.isFile()) throw new IllegalArgumentException("db file '"+db+"' is not a file");
		this.db = db;
	}
	
	
	/**
	 * Returns an initialized {@link ScribbleMgr}.
	 * 
	 * @return a ScribbleMgr that is already connected to the database file
	 */
	protected ScribbleMgr getScribbleMgr(){
		ScribbleMgr sm = new ScribbleMgr();
		try {
			sm.init(this.db);
		} catch (Exception ex) {
			LoggerMgr.getLogger("pdfdoc").log(ex, Level.WARNING);
		}
		return sm;
	}
	
	/**
	 * Return the file name as it is present in the database file.
	 * 
	 * Subclasses implement the correct format of the {@code file_metadata.filename}
	 * column.
	 * 
	 * @return the name of the file as written in the database
	 * 
	 * @see it.andynaz.irex.dr1000.document.ScribbleMgr
	 */
	protected abstract String getFileName();
	
	/**
	 * Return the default filename used to create the output PDF file.
	 * 
	 * The filemane returned should be without the file extensions (it will be
	 * added in the calling methods).
	 * 
	 * @return the filename used to create the output PDF file, when no one is
	 * specified
	 */
	protected abstract String getDefaultFileName();
	
	/**
	 * Creates the output PDF file, using the default color for the strokes (as
	 * specified in the {@link ColorMgr} class).
	 * 
	 * @param outFilePath the path to the output PDF file; if it is {@code null}
	 * or empty, the default one returned by {@link #getDefaultFileName()} is
	 * used
	 */
	public void merge(String outFilePath) {
		merge(outFilePath, ColorMgr.defaultColor);
	}
	
	/**
	 * Creates the output PDF file.
	 * 
	 * @param outFilePath the path to the output PDF file; if it is {@code null}
	 * or empty, the default one returned by {@link #getDefaultFileName()} is
	 * used
	 * @param c strokes color
	 */
	public abstract void merge(String outFilePath, Color c);

	/**
	 * Create a file with all the scribbles in a text format.
	 *
	 * @param outputFile path to the output text file; if it is {@code null}
	 * or empty, the default one returned by {@link #getDefaultFileName()} is
	 * used
	 *
	 * @deprecated only for debug purpuse; will be deleted in future release
	 */
	public void format(String outputFile){
		if (Utils.isEmpty(outputFile))
			outputFile = getDefaultFileName()+".dat";
		
		BufferedWriter w = null;
		
		try{
			ScribbleMgr sm = getScribbleMgr();

			int idFile = sm.getFileId(getFileName());
			if (idFile==0) return;

			List<Integer> pages = sm.getScribbledPages(idFile);
			Collections.sort(pages);

			w = new BufferedWriter(new FileWriter(outputFile));
			w.write("file="+getFileName()); w.newLine();

			for (Integer page : pages){
				w.write("page="+page); w.newLine();
				byte[] blob = sm.getScribbles(page, idFile);
				if(blob==null) continue;
				
				ByteArrayInputStream bIn = new ByteArrayInputStream(blob);
				// ----- page -----																								// ----- page -----
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read())+"\n");                               //2: version
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");       //4: orientation
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read(),
				                             bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");       //8: page pixel
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read())+"\n");                               //2: color R
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read())+"\n");                               //2: color G
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read())+"\n");                               //2: color B
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");       //4: ul_x
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");       //4: ul_y
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");       //4: lr_x
				w.write(Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");       //4: lr_y
																																// ________________________
																																//     36: useless things

				int nStroke = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());	      //4: number of strokes
				w.write(nStroke+"\n");

				// ----- stroke -----																						// ----- stroke -----
				for (int c=0; c<nStroke; c++){
					String t = "\t";
					w.write(t+Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");  //4: color
					w.write(t+Util.get4ByteFloat(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");	   //4: zoom
					w.write(t+Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");  //4: layer
					w.write(t+Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");  //4: shape
					w.write(t+Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read())+"\n");  //4: width

					int nPoint = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());     //4: number of points
					w.write(t+nPoint+"\n");

					// ----- point -----																							// ----- point -----
					for (int j=0; j<nPoint; j++){
						int x = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());	       //4: x
						int y = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());	       //4: y
						int p = Util.getIntFromBytes(bIn.read(), bIn.read(), bIn.read(), bIn.read());	       //4: pressure(?)
						w.write(t+t+x);
						w.write(t+t+t+y);
						w.write(t+t+t+p);
						w.write("\n");

					}
				}
			}

		} catch(Exception e){
			e.printStackTrace();
		} finally{
			if (w!=null){
				try{
					w.close();
				} catch(IOException e){}
			}
		}
	}
	
}
