/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.document;

import java.awt.Color;
import java.io.*;
import java.util.*;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.*;
import it.andynaz.irex.dr1000.scribble.*;
import it.andynaz.log.*;
import it.andynaz.utils.Utils;


/**
 * Represent a DR1000 PDF file.
 * 
 * An object of that kind is associated with a path to a PDF file.
 *
 * @author andynaz
 * @version 2013-08-01
 */
public class PdfDocument extends Document{
	
	/** Name of the default SQLite database file for PDF documents. */
	public static final String DB_DEFAULT_NAME = "metadata.db";
	
	/** Source PDF file. */
	private File file;
	
	
	/**
	 * Creates a PDF document.
	 * 
	 * @param filePath path to the PDF file
	 * 
	 * @throws IllegalArgumentException if {@code filePath} does not represent a
	 * file
	 */
	public PdfDocument(File filePath){
		this(filePath, new File(filePath.getParentFile(), DB_DEFAULT_NAME));
	}
	
	/**
	 * Creates a PDF document.
	 * 
	 * @param file PDF file
	 * @param db database file
	 * 
	 * @throws IllegalArgumentException if {@code file} does not represent a file
	 */
	public PdfDocument(File file, File db){
		super(db);
		if (!file.isFile())
			throw new IllegalArgumentException("error: input file is not a file, filePath="+file);
		this.file = file;
	}
	

	/**
	 * Return the name of the PDF file.
	 *
	 * @return the name of the PDF file
	 */
	@Override
	protected String getFileName(){
		return this.file.getName();
	}
	
	/**
	 * Return the name of the PDF source file, without the extension.
	 * 
	 * @return the name of the PDF source file, without the extension
	 */
	@Override
	protected String getDefaultFileName(){
		String fileName = this.file.getName();
		return (fileName.indexOf('.')==-1 ? fileName : fileName.substring(0, fileName.lastIndexOf('.')))+"__NOTED";
	}

	/**
	 * Merge the scribbles with the PDF.
	 * 
	 * The output is written in the file specified by the 'output' argument.
	 * 
	 * @param outputFilePath the output file path; if it is {@code null} or
	 * empty, the default one returned by {@link #getDefaultFileName()} is used
	 * @param c stokes color
	 */
	@Override
	public void merge(String outputFilePath, Color c){
		Logger logger = LoggerMgr.getLogger("merge");
		
		if (Utils.isEmpty(outputFilePath))
			outputFilePath = getDefaultFileName()+".pdf";
                
		File outputFile = new File(outputFilePath);
		if (file.equals(outputFile)){
			logger.log("error: input and output file are the same", Level.SEVERE);
			return;
		}
		
		try{
			ScribbleMgr sm = getScribbleMgr();
			int idFile = sm.getFileId(file.getName());
			if (idFile==0) return;

			List<Integer> scribbledPages = sm.getScribbledPages(idFile);
			logger.log("file '"+file.getName()+"' has "+scribbledPages.size()+" scribbled pages");
			if (scribbledPages.isEmpty()) return;
			
			PdfReader reader = new PdfReader(file.getPath());
			PdfStamper stamp = new PdfStamper(reader, new FileOutputStream(outputFilePath));
			for (int pageNum=1; pageNum<=reader.getNumberOfPages(); pageNum++){
				if (scribbledPages.contains(pageNum)){
					byte[] blob = sm.getScribbles(pageNum, idFile);
					Page p = Parser.build(blob);
					PdfContentByte cbOver = stamp.getOverContent(pageNum);
					Rectangle rectCropBox = reader.getCropBox(pageNum);
					int nRotation = reader.getPageRotation(pageNum);
					while (nRotation > 0) {
						rectCropBox = rectCropBox.rotate();
						nRotation -= 90;
					}
					if (p!=null)
						p.drawStrokes(cbOver, rectCropBox.left(), rectCropBox.top(), c);
				}
			}
			stamp.close();
			sm.closeConnection();
		} catch(Exception e){
			LoggerMgr.getLogger("merge").log(e, Level.WARNING);
		}
	}
	
}
