/*
 * Copyright 2013 Andrea "andynaz" Agnesse
 * 
 * This file is part of "iRex DR1000 Merger".
 * 
 * "iRex DR1000 Merger" is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 * 
 * "iRex DR1000 Merger" is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * "iRex DR1000 Merger".  If not, see <http://www.gnu.org/licenses/>.
 * 
 * author e-mail: andynaz@gmail.com
 * author home page: http://andynaz.altervista.org
 * program official page: http://andynaz.altervista.org/Progetti/iRexDR1000Merger.shtm
 */

package it.andynaz.irex.dr1000.document;

import static java.lang.System.out;
import java.io.*;
import java.sql.*;
import java.util.*;
import it.andynaz.log.*;
import it.andynaz.utils.Utils;

/**
 * Class to use the SQLite database file with the scribbles.
 * 
 * <p>All the scribbles taken by an iRex DR1000 are stored in SQLite database
 * files. Of the 4 tables in the database, this program uses only the 2 "{@code
 * file_metadata}" and "{@code annotations}"; in the first there are the data of
 * the files in the folder, while in the latter there are the scribbles.<br/>
 * The 2 tables are defined as:<p>
 * <blockquote><pre>
 *                                          +------------------------+
 * +------------------------+               |      annotations       |
 * |     file_metadata      |               +------------------------+
 * +------------------------+               | annotation_id  INTEGER |
 * | file_id   INTEGER      | &lt;------------ | file_id        INTEGER |
 * | filename  VARCHAR(250) |               | file_position  INTEGER |
 * +------------------------+               | data           BLOB    |
 *                                          | <i>other non-used columns</i> |
 *                                          +------------------------+
 * </pre></blockquote>
 * 
 * <p>The file name is stored in the "{@code file_metadata.filename}" column,
 * while the file id is stored in both the "{@code file_metadata.file_id}" and
 * "{@code annotations.file_id}" columns (this is the join column between the 2
 * tables). The scribbles are stored <i>per-page</i>, where the page number is
 * stored in the "{@code annotations.file_position}" column and the actual
 * scribbles data are stored in the "{@code annotations.data}" column.</p>
 * 
 * <p>In case of note files, the content of the {@code file_metadata} table is a
 * single record "{@code file_id=1 filename=.}", as the file contains only a
 * note. In the case of PDF files the table stores a record per file, as there
 * is a {@code metadata.db} file for each folder.</p>
 * 
 * @author andynaz
 * @version 2013-08-01
 */
class ScribbleMgr {
	
	/** Connection to the scribble database file. */
	protected Connection con;
	
	
	/**
	 * Creates a new ScribbleMgr.
	 */
	protected ScribbleMgr(){}

	
	/**
	 * Initialize the connection to an SQLite database with strokes.
	 *
	 * The SQLite database is stored in a SQLite file.
	 *
	 * @param db path of the folder which contains the database file
	 *
	 * @throws ClassNotFoundException if the class for the JDBC driver is not
	 * found
	 * @throws SQLException if SQL errors occur
	 */
	public void init(File db) throws ClassNotFoundException, SQLException{
		Class.forName("org.sqlite.JDBC");
		LoggerMgr.getLogger("init").log("db="+db.getAbsolutePath(), Level.DEBUG);
		this.con = DriverManager.getConnection("jdbc:sqlite:"+db.toString());
	}
	
	/**
	 * Close the connection to the DB.
	 *
	 * @throws SQLException if SQL errors occur
	 */
	public void closeConnection() throws SQLException{
		if (this.con!=null)
			this.con.close();
	}
	
	/**
	 * Returns the id corresponding to a file name.
	 * 
	 * If the file is not found in the database, the program list all the files
	 * in the database having at least a scribble, so that user can choose which
	 * is the corresponding one. This can happen with a file having a name with
	 * all uppercases character, as the Linux operating system of the DR1000 may
	 * not recognize them properly.
	 * 
	 * @param fileName the file name
	 * @return the id of the file, or 0 if the file is not found
	 */
	public int getFileId(String fileName){
		int id = 0;
		try {
			String query = "SELECT file_id FROM file_metadata WHERE filename=?";
			PreparedStatement pst;
				pst = con.prepareStatement(query);
			pst.setString(1, fileName);
			ResultSet rs = pst.executeQuery();
			if (rs.next())
				id = rs.getInt("file_id");
			rs.close();
			pst.close();
			
			if (id==0){
				LoggerMgr.getLogger("scrbl").log("file '"+fileName+"' not found in database", Level.WARNING);
				id = chooseFileFromDB();
			}
			
		} catch (Exception ex) {
			LoggerMgr.getLogger("scrbl").log(ex, Level.WARNING);
		}
		return id;
	}
	
	
	/**
	 * Let you choose the file in the database.
	 * 
	 * Useful when the filename is not found in the database (usually if the
	 * filename is all uppercase).
	 */
	private int chooseFileFromDB() throws IOException{
		int i = 0;
		out.println("\nfiles (with scribbles) in database are:\n");
		out.println("    |  scr. | ");
		out.println(" id | pages | filename");
		out.println("--------------------------------------------------------------------------------");
		String[][] files = getScribbledFiles(); // TODO return e list(array of
		                                        // "DbEntry{id:int,name:String,nPageWithStroke:int}"
		for (String[] file : files)
			out.format("%3s | %4s  | %s\n", file[0], file[1], file[2]);
		//TODO check id is present
		//TODO check id is a number
		out.print("\nChoose an id (or 0 to exit): ");
		BufferedReader bf = new BufferedReader(new InputStreamReader(System.in));
		i = Utils.parseInt( bf.readLine() );
		return i;
	}
	
	/**
	 * Returns a table with the filenames in the database.
	 * 
	 * Only files with scribbles are returned.
	 * 
	 * @return an array with the name of files in the database that have
	 * scribbles; each element in the array is an array with 3 elements, with
	 * file id, number of scribbles and filename
	 * 
	 * @throws SQLException if SQL errors occour
	 */
	private String[][] getScribbledFiles(){
		List<String> ids = new ArrayList<String>();
		List<String> numScribbles = new ArrayList<String>();
		List<String> names = new ArrayList<String>();
		
		try{
			String query = "SELECT file_id, filename,"
					  + "            (SELECT count(file_position) FROM annotations WHERE file_id=f.file_id) scribbles "
					  + " FROM file_metadata f "
					  + " WHERE scribbles<>0 "
					  + " ORDER BY filename";
			PreparedStatement pst = con.prepareStatement(query);
			ResultSet rs = pst.executeQuery();
			while (rs.next()){
				ids.add(rs.getString("file_id"));
				numScribbles.add(rs.getString("scribbles"));
				names.add(rs.getString("filename"));
			}
			rs.close();
			pst.close();
		} catch(SQLException e){
			LoggerMgr.getLogger("file").log(e, Level.WARNING);
		}
		
		String[][] s = new String[names.size()][3];
		for (int i=0; i<ids.size(); i++){
			s[i] = new String[]{ids.get(i), numScribbles.get(i), names.get(i)};
		}
		return s;
	}

	/**
	 * Get a list of the numbers of the pages with scribbles.
	 *
	 * @param idFile file id (as in the database)
	 * @return a list with the number of the pages that have scribbles
	 * 
	 * @throws SQLException
	 */
	public List<Integer> getScribbledPages(int idFile) throws SQLException{
		List<Integer> l = new ArrayList<Integer>();
		String query = "SELECT file_position "
				  + " FROM annotations "
				  + " WHERE file_id=? "
				  + " ORDER BY file_position";
		PreparedStatement pst = con.prepareStatement(query);
		pst.setInt(1, idFile);
		ResultSet rs = pst.executeQuery();
		while (rs.next())
			l.add(rs.getInt("file_position"));
		rs.close();
		pst.close();
		return l;
	}

	/**
	 * Gets a blob with the scribbles as a list of Integer.
	 *
	 * @param pageNumber the number of the page
	 * @param idFile id of the file (as in the database)
	 * @return the blob as a byte array
	 *
	 * @throws SQLException if SQL errors occour
	 */
	public byte[] getScribbles(int pageNumber, int idFile) throws SQLException{
		byte[] blob = null;
		String query = "SELECT data "
				  + " FROM annotations "
				  + " WHERE file_id=? "
				  + "   AND file_position=?";
		PreparedStatement pst = con.prepareStatement(query);
		pst.setInt(1, idFile);
		pst.setInt(2, pageNumber);
		ResultSet rs = pst.executeQuery();
		if (rs.next())
			blob = rs.getBytes("data");
		rs.close();
		pst.close();
		return blob;
	}
	
}
